#include "stdafx.h"
#include "EnumOutlookEvent.h"
#include "OutlookEvent.h"

// CEnumOutlookEvent

CEnumOutlookEvent::CEnumOutlookEvent(	CArray<LPBYTE>& arrayData,
													COutlookReader* pReader,
													bool bAllEvents,
													INT_PTR nArrayIndex /*= 0*/,
													INT_PTR nDataPtrIndex /*= 0*/,
													UINT nCurrentCount /*= 0*/ )
{
	if ( !bAllEvents )
	{
		// We can rely on the pStream for this info if we are handling all events
		m_arrayDataPtr.Copy( arrayData );
	}

	m_bAllEvents			= bAllEvents;
	m_nDataArrayIndex    = nArrayIndex;
	m_nDataPtrIndex      = nDataPtrIndex;
	m_nCurrentCount      = nCurrentCount; 

	m_nRefCount = 0;
	AddRef();

	m_pReader = pReader;
	m_pReader->AddRef();
}

CEnumOutlookEvent::~CEnumOutlookEvent(void)
{
   m_pReader->Release();
   m_arrayDataPtr.RemoveAll();
}

HRESULT __stdcall CEnumOutlookEvent::QueryInterface( const IID& iid, void** ppI )
{
	if ( !ppI )
	{
		return E_POINTER;
	}

	if ( ( iid == IID_IEnumMoraeEvent ) ||
		( iid == IID_IUnknown ) )
	{
		*ppI = static_cast<IEnumMoraeEvent*>( this );
		(static_cast<IUnknown*>(*ppI))->AddRef();
		return S_OK;
	}
	*ppI = NULL;
	return E_NOINTERFACE;
}

ULONG __stdcall CEnumOutlookEvent::AddRef()
{
	return ::InterlockedIncrement( &m_nRefCount );
}

ULONG __stdcall CEnumOutlookEvent::Release()
{
	ULONG nRet = ::InterlockedDecrement( &m_nRefCount );
	if ( nRet == 0 )
	{
		delete this;
		return 0;
	}
	return nRet;
}

HRESULT __stdcall CEnumOutlookEvent::Next( ULONG nCount, IMoraeEvent** ppIMoraeEvent, ULONG* pFetched )
{
	if ( ( nCount > 1 && pFetched == NULL ) || ( !ppIMoraeEvent ) )
	{
		// This combination of parameters is not allowed
		return E_INVALIDARG;
	}
	if ( pFetched )
	{
		*pFetched = 0;
	}
	LPVOID pData = NULL;
	DWORD dwSize = 0;
	IMoraeEvent* pEvent = NULL;
	for ( INT_PTR i = 0; i < (INT_PTR)nCount; i++ )
	{
		pEvent = NULL;		
		if ( !GetNextRecord( &pData, &dwSize ) )
		{
			// We have reached the end of our data
			return S_FALSE;
		}
		
		pEvent = new COutlookEvent( pData, dwSize, m_pReader );

		if ( pEvent )
		{
			pEvent->QueryInterface( IID_IMoraeEvent, (void**)(ppIMoraeEvent + i) );
			pEvent->Release();
			if ( pFetched )
			{
				(*pFetched)++;
			}
		}
	}
	return S_OK;
}

HRESULT __stdcall CEnumOutlookEvent::Clone( IEnumMoraeEvent** ppIEnumMoraeEvent )
{
	if ( !ppIEnumMoraeEvent  )
	{
		return E_INVALIDARG;
	}

	IEnumMoraeEvent* pEvent = NULL;

	try
	{
		pEvent = new CEnumOutlookEvent(	m_arrayDataPtr,
													m_pReader,
													m_bAllEvents,
													m_nDataArrayIndex, 
													m_nDataPtrIndex,
													m_nCurrentCount );
	}
	catch ( CMemoryException* pE )
	{
		pE->Delete();
		if ( pEvent )
		{
			delete pEvent;
		}
		return E_OUTOFMEMORY;
	}
	*ppIEnumMoraeEvent = pEvent;
	return S_OK;
}

HRESULT __stdcall CEnumOutlookEvent::Skip( ULONG nCount )
{
	INT_PTR nOldDataArrayIndex = m_nDataArrayIndex;
	INT_PTR nOldDataPtrIndex = m_nDataPtrIndex;
	UINT nOldCurrentCount = m_nCurrentCount;
	for ( ULONG i = 0; i < nCount; i++ )
	{
		if ( !GetNextRecord( NULL, NULL ) )
		{
			m_nDataArrayIndex = nOldDataArrayIndex;
			m_nDataPtrIndex = nOldDataPtrIndex;
			m_nCurrentCount = nOldCurrentCount;
			return E_FAIL;
		}
	}
	return S_OK;
}

HRESULT __stdcall CEnumOutlookEvent::Reset( void )
{
	m_nDataArrayIndex = 0;
	m_nDataPtrIndex = 0;
	m_nCurrentCount = 0;
	return S_OK;
}

// How this does its job depends on the value of m_bAllEvents.
// If m_pAllEvents is true, the m_arrayDataPtr is an array of the data pointers directly
// created from reading each file.  There will be one in each file.  This function will 
// walk along the data to find the next record.  m_nDataArrayIndex will be the index into
// this array of pointers and m_nDataPtrIndex will give the current location along the 
// current data buffer.
//
// In the case where m_bAllEvents is false, m_arrayDataPtr is an array of pointers, each
// of which points to a single event data structure.  m_nDataPtrIndex will be the index
// into this array.
bool CEnumOutlookEvent::GetNextRecord( LPVOID* ppData, LPDWORD pdwSize )
{
	if ( m_bAllEvents )
	{
		return m_pReader->NextEvent( ppData, pdwSize, &m_nDataArrayIndex, &m_nDataPtrIndex,
			&m_nCurrentCount );
	}

	if ( m_nDataPtrIndex > m_arrayDataPtr.GetUpperBound() )
	{
		return false;
	}

	LPBYTE pData = NULL;
	pData = m_arrayDataPtr.GetAt( m_nDataPtrIndex );

	if ( pdwSize )
	{
		*pdwSize = sizeof(SOutlookMessage);
	}
	if ( ppData )
	{
		*ppData = pData;
	}
	m_nDataPtrIndex ++;
	m_nCurrentCount++;
	return true;
}