#include "stdafx.h"
#include "OutlookReader.h"
#include "OutlookEvent.h"

COutlookEvent::COutlookEvent( LPVOID pData, DWORD dwBytes, COutlookReader* pReader )
: m_pData( pData )
, m_dwDataBytes( dwBytes )
, m_pReader( pReader )
{
	m_nRefCount = 0;
	AddRef();
	// Call AddRef on the reader so it cannot be unloaded while I am using the data
	m_pReader->AddRef();
}

COutlookEvent::~COutlookEvent()
{
	m_pReader->Release();
}

HRESULT __stdcall COutlookEvent::QueryInterface( const IID& iid, void** ppI )
{
	if( !ppI )
	{
		return E_POINTER;
	}

	if ( ( iid == IID_IMoraeEvent ) )
	{
		*ppI = static_cast<IMoraeEvent*>( this );
		(static_cast<IUnknown*>(*ppI))->AddRef();
		return S_OK;
	}

	if ( ( iid == IID_IMoraeSearchableEvent ) )
	{
		*ppI = static_cast/*dynamic_cast*/<IMoraeSearchableEvent*>( this );
		(static_cast<IUnknown*>(*ppI))->AddRef();
		return S_OK;
	}

	*ppI = NULL;
	return E_NOINTERFACE;
}

ULONG __stdcall COutlookEvent::AddRef()
{
	return ::InterlockedIncrement( &m_nRefCount );
}

ULONG __stdcall COutlookEvent::Release()
{
	ULONG nRet = ::InterlockedDecrement( &m_nRefCount );
	if( nRet == 0 )
	{
		delete this;
		return 0;
	}
	return nRet;
}

HRESULT __stdcall COutlookEvent::GetTime( VARIANT* pVarET, VARIANT* pVarTimeDate )
{
	if( m_dwDataBytes < sizeof( SOutlookMessage ) )
	{
		return E_FAIL;
	}

	// Calculate the elapsed time into the video
	double dblStart = 0.0;
	DWORD dwET = ( static_cast<SOutlookMessage*>( m_pData ) )->dwTickCount - m_pReader->GetStartTime( &dblStart );

	if ( pVarET )
	{
		pVarET->vt = VT_I8;
		pVarET->llVal = REF_TIME( dwET );	// TODO: should also handle pause time, not currently implemented in plugin
	}

	if ( pVarTimeDate )
	{
		// Calculate the offset for the timedate
		pVarTimeDate->vt = VT_DATE;
		// This calculation adds the elapsed time in fraction of a day.
		// The paused time is not taken into account in this calculation to get the time of day.
		pVarTimeDate->date = dblStart + (double)dwET / 86400000.0;
	}
	return S_OK;
}

HRESULT __stdcall COutlookEvent::GetTimeRange( VARIANT* pVarStartTimes, VARIANT* pVarEndTimes )
{
	if( m_dwDataBytes < sizeof( SOutlookMessage ) )
	{
		return E_FAIL;
	}

	// Calculate the elapsed time into the video
	double dblStart = 0.0;
	DWORD dwET = ( static_cast<SOutlookMessage*>( m_pData ) )->dwTickCount - m_pReader->GetStartTime( &dblStart );

	HRESULT hRes = E_POINTER;
	// Make sure that the end times is empty
	if ( pVarEndTimes )
	{
		::VariantInit( pVarEndTimes );
		hRes = S_OK;
	}
	if ( pVarStartTimes )
	{
		// Create an array of start times, one element in length
		SAFEARRAY* pArray = NULL;
		if ( !SUCCEEDED( ::SafeArrayAllocDescriptorEx( VT_I8, 1, &pArray ) ) )
		{
			return E_OUTOFMEMORY;
		}

		// Set the descriptor elements
		pArray->rgsabound[0].lLbound = 0;
		pArray->rgsabound[0].cElements = 1;
		pArray->cbElements = sizeof( REFERENCE_TIME );

		// Allocate data for the array
		if ( !SUCCEEDED( ::SafeArrayAllocData( pArray ) ) )
		{
			return E_OUTOFMEMORY;
		}
		long nIndex = 0;
		REFERENCE_TIME rt = REF_TIME( dwET );
		if ( SUCCEEDED( ::SafeArrayPutElement( pArray, &nIndex, &rt ) ) )
		{
			pVarStartTimes->vt = VT_I8 | VT_ARRAY;
			pVarStartTimes->parray = pArray;
			hRes = S_OK;
		}
		else
		{
			::SafeArrayDestroy( pArray );
			return E_OUTOFMEMORY;
		}		
	}
	return hRes;
}

HRESULT __stdcall COutlookEvent::GetAppTag( int* pTag )
{
	return E_NOTIMPL;
}

HRESULT __stdcall COutlookEvent::GetWndClassTag( int* pTag )
{
	return E_NOTIMPL;
}

HRESULT __stdcall COutlookEvent::GetHWND( VARIANT* pVar )
{
	return E_NOTIMPL;
}

HRESULT __stdcall COutlookEvent::GetEventTypeID( UUID* pID )
{
	return m_pReader->GetEventStreamID( pID );
}

HRESULT __stdcall COutlookEvent::GetAttributeString( LCID localeID, int nColID, BSTR* pbstrAttr )
{
	if ( ::IsBadWritePtr( pbstrAttr, sizeof( BSTR ) ) )
	{
		return E_POINTER;
	}
	if( m_dwDataBytes < sizeof( SOutlookMessage ) )
	{
		return E_FAIL;
	}

	BSTR strName = NULL;
	int nStrID = 0;
	int nIndex = m_pReader->GetResultColIndexFromID( nColID );
	bool bRet = false;
	CComBSTR strAttr;
	switch ( nIndex )
	{
		case 0:
		{
			bRet = GetEventTypeString( pbstrAttr );
			break;
		}
		case 1:
		{
			strAttr = static_cast<SOutlookMessage*>(m_pData)->arFrom;
			*pbstrAttr = strAttr.Detach();
			bRet = true;
			break;
		}
		case 2:
		{
			strAttr = static_cast<SOutlookMessage*>(m_pData)->arTo;
			*pbstrAttr = strAttr.Detach();
			bRet = true;
			break;
		}
		case 3:
		{
			strAttr = static_cast<SOutlookMessage*>(m_pData)->arSubject;
			*pbstrAttr = strAttr.Detach();
			bRet = true;
			break;
		}
	}

	return bRet ? S_OK : E_FAIL;
}

HRESULT __stdcall COutlookEvent::GetItemRectangle( int* pLeft, int* pTop, int* pRight, int* pBottom )
{
	*pLeft = *pTop = *pRight = *pBottom = 0;
	return S_OK;
}

bool COutlookEvent::GetEventTypeString( BSTR* pbstrEventName )
{
	DWORD dwEvent = static_cast<SOutlookMessage*>(m_pData)->dwEventType;

	CComBSTR strName;
	bool bRet = false;
	switch ( dwEvent )
	{
		case 0x00000000:
		{
			strName = "Send";
			bRet = true;
			break;
		}
		case 0x00000001:
		{
			strName = "Read";
			bRet = true;
			break;
		}
	}

	*pbstrEventName = strName.Detach();

	return bRet;
}