// OutlookReader.cpp : Implementation of COutlookReader

#include "stdafx.h"
#include "OutlookReader.h"
#include "EnumOutlookEvent.h"
#include <initguid.h> // required to generate a GUID from a DEFINE_GUID macro

// {DFD3C768-0920-4062-9FED-29BE328F9522}
DEFINE_GUID( IID_OUTLOOKSTREAM, 
				0xdfd3c768, 0x920, 0x4062, 0x9f, 0xed, 0x29, 0xbe, 0x32, 0x8f, 0x95, 0x22);

#define NUM_OF_ATTR 4

// COutlookReader
COutlookReader::COutlookReader()
   : m_pIMoraeRecording(NULL)
   , m_pIMoraeProject(NULL)
{
}

COutlookReader::~COutlookReader()
{
   for ( INT_PTR i = 0; i < m_arrayDataPtr.GetSize(); i++ )
   {
      delete m_arrayDataPtr.GetAt( i );
   }
   m_arrayDataPtr.RemoveAll();
   m_arrayEventCount.RemoveAll();
}


HRESULT __stdcall COutlookReader::EnumEvents( /* [out] */ IEnumMoraeEvent **ppIEnum)
{
	CEnumOutlookEvent* pEnum = new CEnumOutlookEvent( m_arrayDataPtr, this, true );
	if( !pEnum )
	{
		return E_OUTOFMEMORY;
	}
	*ppIEnum = pEnum;
	return S_OK;
}

HRESULT __stdcall COutlookReader::GetEventCount( /* [ref][out] */ ULONG *pCount)
{
   *pCount = 0;
   for ( INT_PTR i = 0; i < m_arrayEventCount.GetCount(); i++ )
   {
      (*pCount) += m_arrayEventCount.GetAt( i );
   }
	return S_OK;
}

HRESULT __stdcall COutlookReader::GetEventStreamID( /* [ref][out] */ IID *pID)
{
	*pID = IID_OUTLOOKSTREAM;
	return S_OK;
}

HRESULT __stdcall COutlookReader::SetRecording( /* [in] */ IMoraeRecording *pRecording)
{
	if ( m_pIMoraeRecording )
	{
		m_pIMoraeRecording->Release();
	}
	m_pIMoraeRecording = pRecording;
	if ( m_pIMoraeRecording )
	{
		m_pIMoraeRecording->AddRef();
	}
	return S_OK;
}

HRESULT __stdcall COutlookReader::SetProject( /* [in] */ IMoraeProject *pProject)
{
	if ( m_pIMoraeProject )
	{
		m_pIMoraeProject->Release();
	}

	m_pIMoraeProject = pProject;
	if ( m_pIMoraeProject )
	{
		m_pIMoraeProject->AddRef();
	}
	return S_OK;
}

HRESULT __stdcall COutlookReader::ReadEventFile( /* [in] */ BSTR bstrFile)
{
	CFile dataFile;
	if( !dataFile.Open( bstrFile, CFile::typeBinary|CFile::modeRead ) )
	{
		return E_FAIL;
	}

	ULONGLONG nUnreadBytes = dataFile.GetLength();
	if( nUnreadBytes >= sizeof( DWORD ) )
	{
		OutlookReaderHeader fileHeader;
		nUnreadBytes -= dataFile.Read( &fileHeader, sizeof( OutlookReaderHeader ) );
		
		if( fileHeader.dwVersion > OUTLOOKREADER_CURRENT_VERSION )
		{
			return E_FAIL;
		}

		// Allocate a buffer for the file's contents
		LPBYTE pData = new BYTE[(UINT)nUnreadBytes];
		if( !pData )
		{
			return E_OUTOFMEMORY;
		}
		if( nUnreadBytes != dataFile.Read( pData, (UINT)nUnreadBytes ) )
		{
			return E_FAIL;
		}
      if ( AddFileSet( pData, fileHeader.unNumRecords, fileHeader.dwVersion ) )
      {
         return S_OK;
      }
	}
	return E_FAIL;
}

HRESULT __stdcall COutlookReader::GetStreamName( /* [in] */ LCID localeID, /* [out] */ BSTR *pbstrName)
{
	CComBSTR strName;
	strName = "Outlook Reader";
	*pbstrName = strName.Detach();
	return S_OK;
}

HRESULT __stdcall COutlookReader::SetTempDirectory( 
	/* [in] */ BSTR bstrDir)
{
	return S_OK;
}

HRESULT __stdcall COutlookReader::SetStartTime( /* [in] */ VARIANT varTickCount, /* [in] */ VARIANT varSysTime)
{
	// Store the tick count
	DWORD dwTickCount = 0;
	switch ( varTickCount.vt )
	{
	case VT_I4:
		{
			dwTickCount = (DWORD)varTickCount.lVal;
			break;
		}
	case VT_I8:
		{
			dwTickCount = (DWORD)varTickCount.llVal;
			break;
		}
	case VT_UI4:
		{
			dwTickCount = (DWORD)varTickCount.ulVal;
			break;
		}
	case VT_UI8:
		{
			dwTickCount = (DWORD)varTickCount.ullVal;
			break;
		}
	default:
		{
			return E_INVALIDARG;
		}
	}
	m_dwRecStartTime = dwTickCount;
	if ( varSysTime.vt != VT_DATE )
	{
		return E_INVALIDARG;
	}
	m_dblStartTimeDate = varSysTime.date;
	return S_OK;
}

HRESULT __stdcall COutlookReader::ShowSearchSetupDialog( /* [in] */ LCID localeID, /* [in] */ VARIANT varWndParent)
{
   HRESULT hRet = S_FALSE;
   HWND hWnd = NULL;
   switch ( varWndParent.vt )
   {
   case VT_I4:
      {
         hWnd = (HWND)LongToHandle( varWndParent.lVal );
         break;
      }
   case VT_I8:
      {
         hWnd = (HWND)varWndParent.llVal;
         break;
      }
   }

   COutlookReaderDlg dlg;
   dlg.SetSearchParameters( m_searchParameters );

   if ( IDOK == dlg.DoModal( hWnd ) )
   {
      m_searchParameters = dlg.GetSearchParameters();
      m_bAllowAnyEvent = ( m_searchParameters.bRead && m_searchParameters.bSend == TRUE );
      return S_OK;
   }

   return S_FALSE;
}

HRESULT __stdcall COutlookReader::SetSearchCriteria( /* [in] */ const LPSAFEARRAY pCriteria)
{
   if ( !pCriteria )
      return SetDefaultCriteria();

   // check the array type -should be VT_UI1 - (a byte array)
   VARTYPE vt = VT_EMPTY;
   SAFEARRAY* pArray = const_cast<SAFEARRAY*>( pCriteria );
   if ( S_OK != ::SafeArrayGetVartype( pArray, &vt ) )
      return E_INVALIDARG;

   if ( vt != VT_UI1 )
      return E_INVALIDARG;

   // should be 1 dimensional array
   if ( 1 != ::SafeArrayGetDim( pArray ) )
      return E_INVALIDARG;

   // get the size of the array
   long lLBound = 0;
   if ( S_OK != ::SafeArrayGetLBound( pArray, 1, &lLBound ) )
      return E_INVALIDARG;

   long lUBound = 0;
   if ( S_OK != ::SafeArrayGetUBound( pArray, 1, &lUBound ) )
      return E_INVALIDARG;

   // make sure that we can at least read a version number
   long lArrayBytes = lUBound - lLBound + 1;
   if ( lArrayBytes < sizeof( DWORD ) )
      return E_INVALIDARG;

   // read in the parameters
   HRESULT hRes = S_OK;
   LPBYTE  pBytes = NULL;
   if ( S_OK != ::SafeArrayAccessData( pArray, (void**)&pBytes ) )
      return E_INVALIDARG;

   if ( ::IsBadReadPtr( pBytes, lArrayBytes ) )
      hRes = E_POINTER;
   else
   {
      BSTRBLOB searchBlob;
      searchBlob.cbSize = lArrayBytes;
      searchBlob.pData = pBytes;

      if ( searchBlob.cbSize >= sizeof(DWORD) + sizeof(m_searchParameters) )
      {
         ULONG ulBytes = searchBlob.cbSize;
         LPBYTE pBytes = searchBlob.pData;

         LPDWORD pVersion = reinterpret_cast<LPDWORD>(pBytes);

         if( *pVersion == OUTLOOKREADER_CURRENT_VERSION )
         {
            // we only support one version so far
            // if we change our data, we need to change the version number as well
            // and update this function if we intend to support older blobs

            // offset by the size of the version
            pBytes  += sizeof(DWORD);
            ulBytes -= sizeof(DWORD);

            // we're reading into a temporary parameters object
            // in case we fail - then we can bail out without having changed
            // ourselves at all
            SOutlookReaderCfg tmpParams;

            // read in the marker flag
            if ( ulBytes == sizeof(SOutlookReaderCfg) )
            {
               SOutlookReaderCfg* pSearchParams = reinterpret_cast<SOutlookReaderCfg*>(pBytes);
               tmpParams = *pSearchParams;
               pBytes += sizeof(SOutlookReaderCfg);
               ulBytes -= sizeof(SOutlookReaderCfg);

               ATLASSERT( ulBytes == 0 );
               if ( ulBytes == 0 )
               {
                  // we've read everything in successfully, so let's
                  // go ahead and copy over the parameters
                  m_searchParameters = tmpParams;
                  m_bAllowAnyEvent = ( m_searchParameters.bRead && m_searchParameters.bSend == TRUE );
               }
               else
               {
                  hRes = E_INVALIDARG;
               }
            }
         }
         else
         {
            hRes = E_INVALIDARG;
         }
      }
   }

   ::SafeArrayUnaccessData( pArray );
   return hRes;
}

HRESULT __stdcall COutlookReader::SetDefaultCriteria( void)
{
   m_searchParameters.bRead = TRUE;
   m_searchParameters.bSend = TRUE;

   m_bAllowAnyEvent = true;
	return S_OK;
}

HRESULT __stdcall COutlookReader::GetSearchCriteria( /* [out] */ LPSAFEARRAY *ppBinaryData)
{
   BSTRBLOB searchBlob;
   ::ZeroMemory( &searchBlob, sizeof(searchBlob) );

   HRESULT hRes = S_FALSE;

   searchBlob.cbSize = sizeof(DWORD) + sizeof(m_searchParameters);
   searchBlob.pData = new BYTE[searchBlob.cbSize];

   ULONG ulBytes = searchBlob.cbSize;
   LPBYTE pBytes = searchBlob.pData;

   // write out the version
   LPDWORD pVersion = reinterpret_cast<LPDWORD>(pBytes);
   *pVersion = OUTLOOKREADER_CURRENT_VERSION;
   pBytes  += sizeof(DWORD);
   ulBytes -= sizeof(DWORD);

   // write out the search params
   if ( ulBytes < sizeof(BOOL) )
      return S_FALSE;

   SOutlookReaderCfg* pSearchParams = reinterpret_cast<SOutlookReaderCfg*>(pBytes);
   *pSearchParams = m_searchParameters;
   pBytes += sizeof(SOutlookReaderCfg);
   ulBytes -= sizeof(SOutlookReaderCfg);

   ATLASSERT( ulBytes == 0 );
   ATLASSERT( searchBlob.cbSize > 0 );

   // Create the safe-array
   hRes = E_OUTOFMEMORY;
   SAFEARRAY* pArray = NULL;
   if ( S_OK == ::SafeArrayAllocDescriptorEx( VT_UI1, 1, &pArray ) )
   {
      // allocate data for the array
      pArray->rgsabound[0].lLbound = 0;
      pArray->rgsabound[0].cElements = searchBlob.cbSize;
      pArray->cbElements = sizeof( BYTE );
      if ( S_OK == ::SafeArrayAllocData( pArray ) )
      {
         // Get a pointer to the array data
         LPBYTE pData = NULL;
         if ( S_OK == ::SafeArrayAccessData( pArray, (void**)&pData ) )
         {
            ::CopyMemory( pData, searchBlob.pData, searchBlob.cbSize );
            ::SafeArrayUnaccessData( pArray );
            *ppBinaryData = pArray;
            hRes = S_OK;
         }
      }

      // release the array
      if ( S_OK != hRes )
         ::SafeArrayDestroy( pArray );
   }

   delete[] searchBlob.pData;
   searchBlob.pData = NULL;
   searchBlob.cbSize = 0;

   return hRes;
}

HRESULT __stdcall COutlookReader::RunSearch( /* [out] */ IEnumMoraeEvent **ppEnum, /* [ref][out] */ ULONG *pFoundCount)
{
	// Check pointers
	if ( !ppEnum || !pFoundCount )
	{
		return E_POINTER;
	}
	// Are we filtering?
	if ( m_bAllowAnyTime && m_bAllowAnyEvent )
	{
		// Create an enumerator with all events
		HRESULT hRes = EnumEvents( ppEnum );
		if ( hRes == S_OK )
		{
			GetEventCount( pFoundCount );
		}
		return hRes;
	}

	// Count the events that fit
	ULONG nCount = SearchEvents( );
	if ( nCount == INVALID_DWORD )
	{
		return E_OUTOFMEMORY;
	}

	// If all events are included in search, create a simple enumerator
	ULONG nTotalCount = 0;
	HRESULT hRes = GetEventCount( &nTotalCount );
	if ( hRes != S_OK )
	{
		return hRes;
	}
	if ( nCount == nTotalCount )
	{
		// Create an enumerator with all events
		HRESULT hRes = EnumEvents( ppEnum );
		if ( hRes == S_OK )
		{
			*pFoundCount = nCount;
		}
		return hRes;
	}

	// If not, initialize an array to hold the event pointers and run the search
	CArray<LPBYTE> ptrArray;
	try
	{
		ptrArray.SetSize( nCount );
	}
	catch ( CMemoryException* pE )
	{
		pE->Delete();
		return E_OUTOFMEMORY;
	}
	if ( INVALID_DWORD == SearchEvents( &ptrArray ) )
	{
		return E_OUTOFMEMORY;
	}
	
	// Create an enumerator for the search results
	CEnumOutlookEvent* pEnum = new CEnumOutlookEvent( ptrArray, this, false );
	if ( !pEnum )
	{
		return E_OUTOFMEMORY;
	}
	*ppEnum = pEnum;
	*pFoundCount = nCount;
	return S_OK;
}

HRESULT __stdcall COutlookReader::GetSearchCriteriaString( /* [in] */ LCID localeID, /* [out] */ BSTR *pbstrCriteria)
{
	CComBSTR strName;
	
   if( m_bAllowAnyEvent )
   {
      strName = "All Events";
   }
   else if( m_searchParameters.bSend == FALSE && m_searchParameters.bRead == FALSE )
   {
      strName = "No Events";
   }
   else
   {
      strName = "Specific Events";
   }
   
   *pbstrCriteria = strName.Detach();
	return S_OK;
}

HRESULT __stdcall COutlookReader::SetSearchTimeRange( /* [in] */ VARIANT_BOOL bAllowAll, /* [in] */ VARIANT varStartTimes, /* [in] */ VARIANT varEndTimes)
{
	// Make sure that this is in a form that I understand
	if ( bAllowAll == VARIANT_TRUE )
	{
		m_bAllowAnyTime = true;
		return S_OK;
	}
	
	m_bAllowAnyTime = false;

	// I expect 1-D arrays of 64-bit values
	if ( varStartTimes.vt != ( VT_I8 | VT_ARRAY ) )
	{
		return E_INVALIDARG;
	}
	if ( varEndTimes.vt != ( VT_I8 | VT_ARRAY ) )
	{
		return E_INVALIDARG;
	}

	// Verify data types in the SAFEARRAYs
	VARTYPE vt = VT_EMPTY;
	SAFEARRAY* parStart = varStartTimes.parray;
	SAFEARRAY* parEnd   = varEndTimes.parray;
	::SafeArrayGetVartype( parStart, &vt );
	if ( !( vt == VT_I8 ) )
	{
		return E_INVALIDARG;
	}
	::SafeArrayGetVartype( parEnd, &vt );
	if ( !( vt == VT_I8 ) )
	{
		return E_INVALIDARG;
	}

	// Verify Dimension counts in the SAFEARRAYs
	if ( !( 1 == ::SafeArrayGetDim( parStart ) ) )
	{
		return E_INVALIDARG;
	}
	if ( !( 1 == ::SafeArrayGetDim( parEnd ) ) )
	{
		return E_INVALIDARG;
	}

	// Get the bounds of the arrays
	long nLBoundStart = 0;
	long nUBoundStart = 0;
	if ( !SUCCEEDED( ::SafeArrayGetLBound( parStart, 1, &nLBoundStart ) ) )
	{
		return E_FAIL;
	}
	if ( !SUCCEEDED( ::SafeArrayGetUBound( parStart, 1, &nUBoundStart ) ) )
	{
		return E_FAIL;
	}
	long nLBoundEnd = 0;
	long nUBoundEnd = 0;
	if ( !SUCCEEDED( ::SafeArrayGetLBound( parEnd, 1, &nLBoundEnd ) ) )
	{
		return E_FAIL;
	}
	if ( !SUCCEEDED( ::SafeArrayGetUBound( parEnd, 1, &nUBoundEnd ) ) )
	{
		return E_FAIL;
	}
	// Verify that the two arrays have equal upper and lower bounds
	if ( !( ( nLBoundStart == nLBoundEnd ) && ( nUBoundStart == nUBoundEnd ) ) )
	{
		return E_INVALIDARG;
	}

	// Resize the current array
	try
	{
		m_arrayTimes.SetSize( nUBoundEnd - nLBoundEnd + 1 );
	}
	catch ( CMemoryException* pE )
	{
		pE->Delete();
		return E_OUTOFMEMORY;
	}

	// Get a pointer to the data in each array
	REFERENCE_TIME* pStartTimes = NULL;
	REFERENCE_TIME* pEndTimes   = NULL;
	HRESULT hRes = ::SafeArrayAccessData( parStart, (void**)&pStartTimes );
	if ( !SUCCEEDED( hRes ) )
	{
		return hRes;
	}
	hRes = ::SafeArrayAccessData( parEnd, (void**)&pEndTimes );
	if ( !SUCCEEDED( hRes ) )
	{
		::SafeArrayUnaccessData( parStart );
		return hRes;
	}

	// Set the array values
	// Note that the REFEENCE_TIME values must be translated into tick counts for 
	// efficient searching.
	// We offset these times by the m_dwRecStartTime so we can directly compare these to 
	// the tick count for the event.
	for ( INT_PTR i = 0; i < m_arrayTimes.GetSize(); i++ )
	{
		m_arrayTimes[i].m_dwStart = MS_TIME( *(pStartTimes + i) ) + GetStartTime( NULL );
		m_arrayTimes[i].m_dwEnd   = MS_TIME( *(pEndTimes + i) ) + GetStartTime( NULL );
	}

	::SafeArrayUnaccessData( parStart );
	::SafeArrayUnaccessData( parEnd );
	return S_OK;
}

HRESULT __stdcall COutlookReader::SetSearchApplications( /* [in] */ VARIANT_BOOL bAllowAll, /* [in] */ VARIANT varApps)
{
	return S_OK;
}

HRESULT __stdcall COutlookReader::GetItemAttributeCount( /* [ref][out] */ int *pCount)
{
	*pCount = NUM_OF_ATTR;
   InitializeColIdArray();
	return S_OK;
}

HRESULT __stdcall COutlookReader::GetItemAttributeName( /* [in] */ LCID localeID, /* [in] */ int nIndex, /* [out] */ BSTR *pbstrName)
{
	CComBSTR strName;
	switch( nIndex )
	{
		case 1:
		{
			strName = "Event";
			break;
		}
		case 2:
		{
			strName = "From";
			break;
		}
		case 3:
		{
			strName = "To";
			break;
		}
		case 4:
		{
			strName = "Subject";
			break;
		}
	}
	*pbstrName = strName.Detach();
	return S_OK;
}

HRESULT __stdcall COutlookReader::SetItemAttributeDisplayColID( /* [in] */ int nIndex, /* [in] */ int nColID)
{
   // must subtract 1 from the index, because indexes passed to us
   // are one-based as opposed to zero-based
	INT_PTR index = (INT_PTR)nIndex - 1;
	if ( ( index < m_arrayColIDs.GetSize() ) && ( index >= 0 ) )
	{
		m_arrayColIDs.SetAt( index, nColID );
		return S_OK;
	}
	return E_INVALIDARG;
}

HRESULT __stdcall COutlookReader::GetName( /* [in] */ LCID localeID, /* [out] */ BSTR *pbstrName)
{
	CComBSTR strName;
	strName = "Outlook Reader";
	*pbstrName = strName.Detach();
	return S_OK;
}

HRESULT __stdcall COutlookReader::GetSearchTypeID( /* [out] */ IID *pID)
{
	*pID = IID_OUTLOOKSTREAM;
	return S_OK;
}

HRESULT __stdcall COutlookReader::HasSetupDialog( /* [ref][out] */ VARIANT_BOOL *pBool)
{
   *pBool = VARIANT_TRUE;
   return S_OK;
}

DWORD COutlookReader::GetStartTime( double* pStartDateTime )
{
	if( pStartDateTime )
	{
		*pStartDateTime = m_dblStartTimeDate;
	}
	return m_dwRecStartTime;
}

// Walks through the event stream, searching for those that meet the search criteria
// If pArray == NULL, this function simply counts the events.
// If pArray != NULL, it sets the values of the pointers
// Returns the number of events found, or INVALID_DWORD if an exception occurs.
ULONG COutlookReader::SearchEvents( CArray<LPBYTE>* pArray /*= NULL */)
{
	// Variables needed for NextEvent call
	LPVOID pEvent = NULL;
	INT_PTR nArrayIndex = 0;
	INT_PTR nDataOffset = 0;
	UINT nEventCount = 0;

	// Count that will be returned
	ULONG nFoundCount = 0;
	INT_PTR nOutputIndex = 0;

	try
	{
		while ( NextEvent( &pEvent, NULL, &nArrayIndex, &nDataOffset, &nEventCount ) )
		{
			if ( CheckEventTime( pEvent ) )
			{
            if( CheckEventType( pEvent ) )
            {
				   nFoundCount++;
				   if( pArray )
				   {
					   if( nOutputIndex < pArray->GetSize() )
					   {
						   pArray->SetAt( nOutputIndex, reinterpret_cast<LPBYTE>(pEvent) );
					   }
					   else
					   {
						   pArray->SetAtGrow( nOutputIndex, reinterpret_cast<LPBYTE>(pEvent) );
					   }
					   nOutputIndex++;
               }
				}
			}
		}
	}
	catch ( CMemoryException* pE )
	{
		pE->Delete();
		return INVALID_DWORD;
	}

	return nFoundCount;
}

// Used to retreive the next record in the event stream.
//
// The last three arguments are in/out.
// The input value should be the value that was output from the previous call.
// These should be set to 0 in the initial call to this function when stepping through the
// events in this stream.
// pDataOffset is the offset into the current m_arrayDataPtr buffer.
// pCurrentCount is the count of records retrieved from the current m_arrayDataPtr buffer.
bool COutlookReader::NextEvent( LPVOID* ppData, LPDWORD pdwSize, INT_PTR* pArrayIndex, INT_PTR* pDataOffset, UINT* pCurrentCount )
{
   if ( *pArrayIndex > m_arrayEventCount.GetUpperBound() )
   {
      // We have reached the end of our data
      return false;
   }
	LPBYTE pData = NULL;
   while ( *pCurrentCount >= m_arrayEventCount.GetAt( *pArrayIndex ) )
   {
      (*pArrayIndex)++;
      if ( *pArrayIndex > m_arrayEventCount.GetUpperBound() )
      {
         // We have reached the end of our data
         return false;
      }
      *pDataOffset = 0;
      *pCurrentCount = 0;
   }
	pData = m_arrayDataPtr.GetAt( *pArrayIndex ) + *pDataOffset;
	// Get the size of this record
	DWORD dwSize = sizeof(SOutlookMessage);;
	DWORD dwDataOffsetIncr = 0;

	dwDataOffsetIncr += dwSize;
	if ( pdwSize )
	{
		*pdwSize = dwSize;
	}
	if ( ppData )
	{
		*ppData = pData;
	}
	*pDataOffset += dwDataOffsetIncr;
	(*pCurrentCount)++;
	return true;
}

// Returns true if this event meets the search criteria
// The index and start inputs are only applicable to tasks
bool COutlookReader::CheckEventTime( LPVOID pEvent )
{
	if ( m_bAllowAnyTime )
	{
		return true;
	}

	DWORD dwEventTime = 0xFFFFFFFF;
	SOutlookMessage* pOutlookEvent = static_cast<SOutlookMessage*>( pEvent );
	dwEventTime = pOutlookEvent->dwTickCount;	// TODO add correction for pause time

	// Note that these times have already been corrected for starting tick count and offset.
	for ( INT_PTR i = 0; i < m_arrayTimes.GetCount(); i++ )
	{
		if ( dwEventTime >= m_arrayTimes.GetAt( i ).m_dwStart &&
			dwEventTime <= m_arrayTimes.GetAt( i ).m_dwEnd )
		{
			return true;
		}
	}
	return false;
}

// Returns true if this event meets the search criteria
// The index and start inputs are only applicable to tasks
bool COutlookReader::CheckEventType( LPVOID pEvent )
{
   DWORD dwEventType = 0xFFFFFFFF;
   SOutlookMessage* pOutlookEvent = static_cast<SOutlookMessage*>( pEvent );
   dwEventType = pOutlookEvent->dwEventType;

   if( m_searchParameters.bSend && dwEventType == SendEvent )
   {
      return true;
   }

   if( m_searchParameters.bRead && dwEventType == ReadMailEvent )
   {
      return true;
   }

   return false;
}

int COutlookReader::GetResultColIndexFromID( int nColID )
{
	int nRet = -1;
	for ( INT_PTR i = 0; i < m_arrayColIDs.GetSize(); i++ )
	{
		if ( nColID == m_arrayColIDs.GetAt( i ) )
		{
			nRet = (int)i;
			break;
		}
	}
	return nRet;
}

bool COutlookReader::InitializeColIdArray()
{
   // Initialize the ColIDs array
   INT_PTR nCount = static_cast<INT_PTR>( NUM_OF_ATTR );

   if ( m_arrayColIDs.GetSize() != nCount )
   {
      try
      {
         m_arrayColIDs.SetSize( nCount );
         for ( INT_PTR i = 0; i < nCount; ++i )
         {
            m_arrayColIDs[i] = -1;
         }
      }
      catch ( CMemoryException* pE )
      {
         pE->Delete();
         return false;
      }
   }

   return true;
}

bool COutlookReader::AddFileSet( LPBYTE pData, UINT nEventCount, int nVersion )
{
   if ( m_arrayDataPtr.GetCount() == 0 )
   {
      // This is the first data set, so store version
      m_nVersion = nVersion;
   }

   // Make sure version is correct
   if (  m_nVersion != nVersion )
   {
      return false;
   }

   try
   {
      m_arrayDataPtr.Add( pData );
      m_arrayEventCount.Add( nEventCount );
   }
   catch( CMemoryException* pE )
   {
      pE->Delete();
      // Make sure that the arrays are the same size
      if ( m_arrayDataPtr.GetCount() > m_arrayEventCount.GetCount() )
      {
         m_arrayDataPtr.RemoveAt( m_arrayDataPtr.GetUpperBound() );
      }
      return false;
   }
   return true;
}