// OutlookReader.h : Declaration of the COutlookReader

#pragma once
#include "resource.h"       // main symbols

#include "OutlookReader_i.h"
#include "OutlookReaderDlg.h"

#define OUTLOOKREADER_CURRENT_VERSION	0x00010000

// Use these as initial values and parameters when you want to use -1
#define INVALID_DWORD static_cast<DWORD>(-1)  // DWORD is unsigned, this is the maximum positive DWORD value

#if defined(_WIN32_WCE) && !defined(_CE_DCOM) && !defined(_CE_ALLOW_SINGLE_THREADED_OBJECTS_IN_MTA)
#error "Single-threaded COM objects are not properly supported on Windows CE platform, such as the Windows Mobile platforms that do not include full DCOM support. Define _CE_ALLOW_SINGLE_THREADED_OBJECTS_IN_MTA to force ATL to support creating single-thread COM object's and allow use of it's single-threaded COM object implementations. The threading model in your rgs file was set to 'Free' as that is the only threading model supported in non DCOM Windows CE platforms."
#endif

// OutlookReaderHeader
struct OutlookReaderHeader
{
	DWORD    dwVersion;
	UINT     unNumRecords;
};

// OutlookMessage
struct SOutlookMessage
{
	DWORD dwEventType;
	DWORD	dwTickCount;
	char	arFrom[128];
	char	arTo[128];
	char	arSubject[128];
};

enum OutlookEventType
{
	SendEvent		= 0x00000000,
	ReadMailEvent	= 0x00000001
};

typedef struct _TIME_INTERVAL
{
	DWORD m_dwStart;
	DWORD m_dwEnd;
}TIME_INTERVAL;

// COutlookReader

class ATL_NO_VTABLE COutlookReader :
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<COutlookReader, &CLSID_OutlookReader>,
	public IMoraeEventStream,
	public IMoraeEventSearch
{
public:
	COutlookReader();

	~COutlookReader();

DECLARE_REGISTRY_RESOURCEID(IDR_OUTLOOKREADER1)

DECLARE_NOT_AGGREGATABLE(COutlookReader)

BEGIN_COM_MAP(COutlookReader)
	COM_INTERFACE_ENTRY(IMoraeEventStream)
	COM_INTERFACE_ENTRY(IMoraeEventSearch)
END_COM_MAP()



	DECLARE_PROTECT_FINAL_CONSTRUCT()

	HRESULT FinalConstruct()
	{
		return S_OK;
	}

	void FinalRelease()
	{
	}

public:
	// Implementation of IMoraeEventStream
	virtual HRESULT __stdcall EnumEvents( IEnumMoraeEvent **ppIEnum);
	virtual HRESULT __stdcall GetEventCount( ULONG *pCount);
	virtual HRESULT __stdcall GetEventStreamID( IID *pID);
	virtual HRESULT __stdcall SetRecording( IMoraeRecording *pRecording);
	virtual HRESULT __stdcall SetProject( IMoraeProject *pProject);
	virtual HRESULT __stdcall ReadEventFile( BSTR bstrFile);
	virtual HRESULT __stdcall GetStreamName( LCID localeID, BSTR *pbstrName);
	virtual HRESULT __stdcall SetTempDirectory( BSTR bstrDir);
	virtual HRESULT __stdcall SetStartTime( VARIANT varTickCount, VARIANT varSysTime);

	// Implementation of IMoraeEventSearch
	virtual HRESULT __stdcall ShowSearchSetupDialog( LCID localeID, VARIANT varWndParent);
	virtual HRESULT __stdcall SetSearchCriteria( const LPSAFEARRAY pCriteria);
	virtual HRESULT __stdcall SetDefaultCriteria( void);
	virtual HRESULT __stdcall GetSearchCriteria( LPSAFEARRAY *ppBinaryData);
	virtual HRESULT __stdcall RunSearch( IEnumMoraeEvent **ppEnum, ULONG *pFoundCount);
	virtual HRESULT __stdcall GetSearchCriteriaString( LCID localeID, BSTR *pbstrCriteria);
	virtual HRESULT __stdcall SetSearchTimeRange( VARIANT_BOOL bAllowAll, VARIANT varStartTimes, VARIANT varEndTimes);
	virtual HRESULT __stdcall SetSearchApplications( VARIANT_BOOL bAllowAll, VARIANT varApps);
	virtual HRESULT __stdcall GetItemAttributeCount( int *pCount);
	virtual HRESULT __stdcall GetItemAttributeName( LCID localeID, int nIndex, BSTR *pbstrName);
	virtual HRESULT __stdcall SetItemAttributeDisplayColID( int nIndex, int nColID);
	virtual HRESULT __stdcall GetName( LCID localeID, BSTR *pbstrName);
	virtual HRESULT __stdcall GetSearchTypeID( IID *pID);
   virtual HRESULT __stdcall HasSetupDialog( VARIANT_BOOL *pBool);

	bool NextEvent( LPVOID* ppData, LPDWORD pdwSize, INT_PTR* pArrayIndex, INT_PTR* pDataOffset, UINT* pCurrentCount );
	DWORD GetStartTime( double* pStartDateTime );
	ULONG SearchEvents( CArray<LPBYTE>* pArray = NULL );
	bool CheckEventTime( LPVOID pEvent );
   bool CheckEventType( LPVOID pEvent );
	int GetResultColIndexFromID( int nColID );
   bool AddFileSet( LPBYTE pData, UINT nEventCount, int nVersion );

protected:
	DWORD								m_nVersion;
   CArray<LPBYTE>             m_arrayDataPtr;
   CArray<UINT>               m_arrayEventCount;

	IMoraeRecording*           m_pIMoraeRecording;	// Used to maintain refcount of recording
	IMoraeProject*             m_pIMoraeProject;
	DWORD                      m_dwRecStartTime;
	double                     m_dblStartTimeDate;
	CArray<int>                m_arrayColIDs;

	bool								m_bAllowAnyTime;
   bool								m_bAllowAnyEvent;
	CArray<TIME_INTERVAL>		m_arrayTimes;
   SOutlookReaderCfg          m_searchParameters;

   bool  InitializeColIdArray();
};

OBJECT_ENTRY_AUTO(__uuidof(OutlookReader), COutlookReader)
