﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using Microsoft.VisualStudio;

namespace MoraeEventCapture
{
   public enum SetupStatus
   {
      Setup_Ready = 1,
      Setup_Incomplete = 2,
   };

   // Enumeration of calibration requirements for GetCalibrationRequirements
   public enum CalibReq
   {
      Calib_Never = 0x00000000,
      Calib_OneTime = 0x00000001,
      Calib_EachRecording = 0x00000002,
      Calib_EachStartup = 0x00000004,
   };

   // Enumeration of calibration states for GetCalibStatus
   public enum CalibStatus
   {
      Calib_Ready = 1,
      Calib_Needed = 2,
   };

   // Enumeration of capture control values for CaptureControl
   public enum CaptureControl
   {
      Capture_Idle = 0,
      Capture_Start = 1,
      Capture_Stop = 2,
      Capture_Pause = 3,
      Capture_Resume = 4,
   };

   // This Guid is IID_IEventCapture
   [Guid( "78F72AF2-41A0-47F3-B753-6139A9849708" )]
   [InterfaceType( ComInterfaceType.InterfaceIsIUnknown )]
   [ComVisible( true )]
   public interface IEventCapture
   {
      [PreserveSig] 
      System.Int32 GetName(
                                 [In, Out, MarshalAs( UnmanagedType.BStr )]ref string pStr,
                                 [In]Int32 nLocaleID );

      [PreserveSig]
      System.Int32 GetDescription(
                                 [In, Out, MarshalAs( UnmanagedType.BStr )]ref string pStr,
                                 [In]Int32 nLocaleID );

      [PreserveSig]
      System.Int32 HasSetupDialog(
                                 [In, Out, MarshalAs( UnmanagedType.VariantBool )]ref bool pBool );

      [PreserveSig]
      System.Int32 HasCalibrationDialog(
                                 [In, Out, MarshalAs( UnmanagedType.VariantBool )]ref bool pBool );

      [PreserveSig]
      System.Int32 GetSetupStatus(
                                 [In, Out, MarshalAs( UnmanagedType.I4 )]ref int pStatus );

      [PreserveSig]
      System.Int32 GetCalibrationStatus(
                                 [In, Out, MarshalAs( UnmanagedType.I4 )]ref int pStatus );

      [PreserveSig]
      System.Int32 GetCalibrationRequirements(
                                 [In, Out, MarshalAs( UnmanagedType.U4 )]ref System.UInt32 pCalReq );

      [PreserveSig]
      System.Int32 GetCaptureParameters(
                                 [In, Out, MarshalAs( UnmanagedType.SafeArray, SafeArraySubType = VarEnum.VT_UI1 )]ref Byte[] pVarParam );

      [PreserveSig]
      System.Int32 SetCaptureParameters(
                                 [In, MarshalAs( UnmanagedType.SafeArray, SafeArraySubType = VarEnum.VT_UI1 )]Byte[] varParam );

      [PreserveSig]
      System.Int32 SetOutputDirectory(
                                 [In, MarshalAs( UnmanagedType.BStr )]string strDir );

      [PreserveSig]
      System.Int32 GetOutputFilepaths(
                                 [In, Out, MarshalAs( UnmanagedType.SafeArray, SafeArraySubType = VarEnum.VT_BSTR )]ref String[] parNames );

      [PreserveSig]
      System.Int32 ShowSetupDialog(
                                 [In, MarshalAs( UnmanagedType.Struct )]Object varHwndParent,
                                 [In]Int32 nLocaleID );

      [PreserveSig]
      System.Int32 ShowCalibrationDialog(
                                 [In, MarshalAs( UnmanagedType.Struct )]Object varHwndParent,
                                 [In]Int32 nLocaleID );

      [PreserveSig]
      System.Int32 SetCaptureRect(
                                 [In]int nTop,
                                 [In]int nBottom,
                                 [In]int nLeft,
                                 [In]int nRight );

      [PreserveSig]
      System.Int32 ControlCapture(
                                 [In]int nCmd,
                                 [In]UInt32 nTime );

      [PreserveSig]
      System.Int32 GetEventReaderInfo(
                                 [In, Out]ref Guid puuidID,
                                 [In, Out]ref UInt32 pnMajorVersion,
                                 [In, Out]ref UInt32 pnMinorVersion,
                                 [In, Out, MarshalAs( UnmanagedType.BStr )]ref string pStrHandlerName,
                                 [In, Out, MarshalAs( UnmanagedType.BStr )]ref string pStrURL,
                                 [In]Int32 nLocaleID );
   }

   // This is a partial implementation of the class that implements the IEventCapture interface.
   // Derived classes should override any of these virtual methods that need to be implemented.
   // The derived class should have Guid and ComVisible attributes in this form:
   //[Guid( "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx" )]
   //[ComVisible( true )]
   public class EventCaptureWrapper : IEventCapture
   {
      virtual public System.Int32 GetName( [In, Out, MarshalAs( UnmanagedType.BStr )]ref string pStr,
                          [In]Int32 nLocaleID )
      {
         return VSConstants.E_NOTIMPL;
      }

      virtual public System.Int32 GetDescription(
                              [In, Out, MarshalAs( UnmanagedType.BStr )]ref string pStr,
                              [In]Int32 nLocaleID )
      {
         return VSConstants.E_NOTIMPL;
      }

      virtual public System.Int32 HasSetupDialog(
                                 [In, Out, MarshalAs( UnmanagedType.VariantBool )]ref bool pBool )
      {
         pBool = false;
         return VSConstants.S_OK;
      }

      virtual public System.Int32 HasCalibrationDialog(
                                 [In, Out, MarshalAs( UnmanagedType.VariantBool )]ref bool pBool )
      {
         pBool = false;
         return VSConstants.S_OK;
      }

      virtual public System.Int32 GetSetupStatus(
                                 [In, Out, MarshalAs( UnmanagedType.I4 )]ref int pStatus )
      {
         pStatus = (int)SetupStatus.Setup_Ready;
         return VSConstants.S_OK;
      }

      virtual public System.Int32 GetCalibrationStatus(
                                 [In, Out, MarshalAs( UnmanagedType.I4 )]ref int pStatus )
      {
         pStatus = (int)CalibStatus.Calib_Ready;
         return VSConstants.S_OK;
      }

      virtual public System.Int32 GetCalibrationRequirements(
                                 [In, Out, MarshalAs( UnmanagedType.U4 )]ref System.UInt32 pCalReq )
      {
         pCalReq = (System.UInt32)CalibReq.Calib_Never;
         return VSConstants.S_OK;
      }

      virtual public System.Int32 GetCaptureParameters(
                                 [In, Out, MarshalAs( UnmanagedType.SafeArray, SafeArraySubType = VarEnum.VT_UI1 )]ref Byte[] parParam )
      {
         return VSConstants.S_FALSE;
      }

      virtual public System.Int32 SetCaptureParameters(
                                 [In, MarshalAs( UnmanagedType.SafeArray, SafeArraySubType = VarEnum.VT_UI1 )]Byte[] arParam )
      {
         return VSConstants.S_OK;
      }

      virtual public System.Int32 SetOutputDirectory(
                                 [In, MarshalAs( UnmanagedType.BStr )]string strDir )
      {
         return VSConstants.S_OK;
      }

      virtual public System.Int32 GetOutputFilepaths(
                                 [In, Out, MarshalAs( UnmanagedType.SafeArray, SafeArraySubType = VarEnum.VT_BSTR )]ref String[] parNames )
      {
         return VSConstants.E_NOTIMPL;
      }

      virtual public System.Int32 ShowSetupDialog(
                                 [In, MarshalAs( UnmanagedType.Struct )]Object varHwndParent,
                                 [In]Int32 nLocaleID )
      {
         // No setup dialog is needed for this plugin

///////////////////////////////////////////////////////////////////////////////////////////
         // This code is an example of how to use the HWND that is supplied by the caller
         //OutlookAddIn1.CWin32Wrapper parent;
         //if ( varHwndParent is long )
         //{
         //   parent = new OutlookAddIn1.CWin32Wrapper( (long)varHwndParent );
         //}
         //else if ( varHwndParent is int )
         //{
         //   parent = new OutlookAddIn1.CWin32Wrapper( (long)((int)varHwndParent) );
         //} 
         //else
         //{
         //   parent = new OutlookAddIn1.CWin32Wrapper( (long)0 );
         //}
         //string msg = "No setup dialog is available";
         //System.Windows.Forms.MessageBox.Show( parent, msg );
///////////////////////////////////////////////////////////////////////////////////////////


         return VSConstants.S_FALSE;
      }

      virtual public System.Int32 ShowCalibrationDialog(
                                 [In, MarshalAs( UnmanagedType.Struct )]Object varHwndParent,
                                 [In]Int32 nLocaleID )
      {
         // No calibration dialog is needed for this plugin
         return VSConstants.S_FALSE;
      }

      virtual public System.Int32 SetCaptureRect(
                                 [In]int nTop,
                                 [In]int nBottom,
                                 [In]int nLeft,
                                 [In]int nRight )
      {
         // For now, capture all emails, ignoring capture rectangle
         return VSConstants.S_OK;
      }

      virtual public System.Int32 ControlCapture(
                                 [In]int nCmd,
                                 [In]UInt32 nTime )
      {
         return VSConstants.S_OK;
      }

      virtual public System.Int32 GetEventReaderInfo(
                                 [In, Out]ref Guid puuidID,
                                 [In, Out]ref UInt32 pnMajorVersion,
                                 [In, Out]ref UInt32 pnMinorVersion,
                                 [In, Out, MarshalAs( UnmanagedType.BStr )]ref string pStrHandlerName,
                                 [In, Out, MarshalAs( UnmanagedType.BStr )]ref string pStrURL,
                                 [In]Int32 nLocaleID )
      {
         return VSConstants.E_NOTIMPL;
      }
   }
}
