﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Linq;
using Outlook = Microsoft.Office.Interop.Outlook;
using Office = Microsoft.Office.Core;
using System.Runtime.InteropServices;
using MoraeEventCapture;

namespace OutlookAddIn1
{
   public partial class ThisAddIn
   {
      const Int32 OUTLOOKREADER_CURRENT_VERSION = 0x00010000;

      [StructLayout(LayoutKind.Sequential)]
      private struct sMessage
      {         
         public UInt32  uEventType;
         public Int32   nTickCount;
         [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 128)]
         public string  strFrom;
         [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 128)]
         public string  strTo;
         [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 128)]
         public string  strSubject;
      }

      [StructLayout(LayoutKind.Sequential)]
      private struct sFileHeader
      {
         public   Int32    nVersion;
         public   Int32    nEventCount; 
      }

      enum OutlookEventType 
      {
         SendEvent      = 0x00000000,
         ReadMailEvent  = 0x00000001
      }

      [DllImport( "kernel32.dll", CharSet = CharSet.Auto, SetLastError = true )]
      public static extern UInt32 GetCurrentProcessId();

      private IPluginManager recHelper;
      private OutlookEventCapture evntCap = new OutlookEventCapture();
      private Guid guidRec;
      private Guid guidHelper;
      private System.IO.FileStream m_outFile = null;
      private System.IO.BinaryWriter m_outWriter = null;
      private Outlook.MailItem m_mailItem;
      private sFileHeader m_FileHeader;

      private void ThisAddIn_Startup(object sender, System.EventArgs e)
      {
         // Initialize the GUIDs that define this object.
         // This is the GUID that we want Morae Recorder to use when it is asking for
         // this object.  When we call IPluginManager::AddEventSource, this is what we
         // pass to it.
         guidRec     = new Guid("6EA26F89-81AF-4a98-9AA1-C73DF667CD5C");
         // This is the GUID that IPluginManager recognizes as this object.  We have to tell
         // IPluginManager about this GUID in the call to IPluginManager::RegisterComponent
         // and IPluginManager::UnregisterComponent.  (These methods should only be called
         // when this component is registered or unregistered during installation.) 
         // Note that this GUID matches the Guid attribute of EventCaptureWrapper.
         guidHelper  = new Guid("325BBAB8-98F2-44af-85BD-383A6FD97C99");

         Outlook.Application outlookApp =
           OutlookAddIn1.Globals.ThisAddIn.Application;

         // Capture email send events
         outlookApp.ItemSend += new Microsoft.Office.Interop.Outlook.ApplicationEvents_11_ItemSendEventHandler( outlookApp_ItemSend );

         // Capture when user opens an inspector window for an email
         outlookApp.Inspectors.NewInspector += new Microsoft.Office.Interop.Outlook.InspectorsEvents_NewInspectorEventHandler(Inspectors_NewInspector);

         // Capture event capture status change events from Recorder
         evntCap.eventCaptureStatus += new CaptureStatusChanged( evntCap_eventCaptureStatus );

         try
         {
            RecHelperManager recCOM = new RecHelperManager();

            recHelper = (IPluginManager)recCOM;
            
            // For testing
            //recHelper.RegisterComponent( "OutlookPlugin", ref guidHelper, ref guidRec );
            /////////////////////////

            recHelper.AddEventSource( evntCap, GetCurrentProcessId(), ref guidRec );
         }
         catch ( System.Runtime.InteropServices.COMException ex )
         {
            HandleCOMException( ex );
         }
         catch ( System.Runtime.InteropServices.SEHException ex )
         {
            HandleCOMException( ex );
         }
         catch ( System.NotImplementedException ex )
         {
            HandleCOMException( ex );
         }
      }

      void Inspectors_NewInspector(Microsoft.Office.Interop.Outlook.Inspector Inspector)
      {
         if (evntCap.GetCaptureStatus() == CaptureControl.Capture_Start)
         {
            m_mailItem = Inspector.CurrentItem as Outlook.MailItem;
            if ( m_mailItem != null)
            {
               // Capture read events for messages opened in this Inspector window
               m_mailItem.Read += new Microsoft.Office.Interop.Outlook.ItemEvents_10_ReadEventHandler(m_mailItem_Read);
            }
         }
      }

      void outlookApp_ItemSend(object Item, ref bool Cancel)
      {
         if (evntCap.GetCaptureStatus() == CaptureControl.Capture_Start)
         {
            Outlook.MailItem mailItem = Item as Outlook.MailItem;
            if (mailItem != null && m_outFile != null)
            {
               m_FileHeader.nEventCount++;

               sMessage msgSend;
               msgSend.uEventType = Convert.ToUInt32(OutlookEventType.SendEvent);
               msgSend.nTickCount = Environment.TickCount;
               msgSend.strFrom = mailItem.SenderName;
               msgSend.strTo = mailItem.To;
               msgSend.strSubject = mailItem.Subject;

               byte[] arBytes = RawSerialize(msgSend);
               m_outWriter.Write(arBytes);
            }
         }
      }

      // Currently only captures read events from new Inspector window, not from Preview pane
      void m_mailItem_Read()
      {
         if (evntCap.GetCaptureStatus() == CaptureControl.Capture_Start)
         {
            if (m_mailItem != null && m_outFile != null)
            {
               m_FileHeader.nEventCount++;

               sMessage msgSend;
               msgSend.uEventType = Convert.ToUInt32(OutlookEventType.ReadMailEvent);
               msgSend.nTickCount = Environment.TickCount;
               msgSend.strFrom = m_mailItem.SenderName;
               msgSend.strTo = m_mailItem.To;
               msgSend.strSubject = m_mailItem.Subject;

               byte[] arBytes = RawSerialize(msgSend);
               m_outWriter.Write(arBytes);
            }
         }
      }

      void evntCap_eventCaptureStatus( object sender, CaptureStatusEventArgs e )
      {
         if ( e.captureCmd == CaptureControl.Capture_Start )
         {
            if ( evntCap.GetOutputDirectory() == null ||
                 evntCap.GetOutputDirectory().Length == 0 )
            {
               System.Windows.Forms.MessageBox.Show( "Output Directory not set!" );
            }
           
            // Do all setup to start capture, etc.
            // For now, set up output file paths
            string strOutDir = evntCap.GetOutputDirectory();
            evntCap.ClearOuputPaths();
            string strOutPath = System.IO.Path.Combine( strOutDir, "OutlookFile.plg" );
            int nCount = 0;
            while ( System.IO.File.Exists( strOutPath ) )
            {
               strOutPath = System.IO.Path.Combine( strOutDir, "OutlookFile" + nCount.ToString() + ".plg" );
               nCount++;
            }

            // Reset event count and version
            m_FileHeader.nEventCount = 0;
            m_FileHeader.nVersion = OUTLOOKREADER_CURRENT_VERSION;

            m_outFile = System.IO.File.Create( strOutPath );
            m_outWriter = new System.IO.BinaryWriter( m_outFile );
            evntCap.AddOutputPath( strOutPath );

            // Write Header info
            // Right now only a place holder
            // Correct header will be written at the end
            byte[] arBytes = RawSerialize(m_FileHeader);
            m_outWriter.Write(arBytes);
         }
         else if ( e.captureCmd == CaptureControl.Capture_Stop )
         {
            CloseEventFile();
         }
      }

      private void ThisAddIn_Shutdown(object sender, System.EventArgs e)
      {
         // Close event file in case we are in the middle of a recording
         CloseEventFile();

         Outlook.Application outlookApp =
           OutlookAddIn1.Globals.ThisAddIn.Application;
         outlookApp.ItemSend -= new Microsoft.Office.Interop.Outlook.ApplicationEvents_11_ItemSendEventHandler( outlookApp_ItemSend );
         
         try
         {
            recHelper.RemoveEventSource( evntCap, GetCurrentProcessId(), ref guidRec );

            // For testing
            //recHelper.UnregisterComponent( "OutlookPlugin", ref guidHelper, ref guidRec );
            /////////////////////////
         }
         catch ( System.Runtime.InteropServices.COMException ex )
         {
            HandleCOMException( ex );
         }
         catch ( System.Runtime.InteropServices.SEHException ex )
         {
            HandleCOMException( ex );
         }
         catch ( System.NotImplementedException ex )
         {
            HandleCOMException( ex );
         }
      }
      private void HandleCOMException( Object objException )
      {
         // This will only show a message for a debug build
         //System.Diagnostics.Debug.Assert( false, objException.ToString() );
         System.Windows.Forms.MessageBox.Show( objException.ToString() );
      }

      //This Function takes any structure, and will return an array of bytes, that can be nicely written to a file via the BinaryWriter
      public static byte[] RawSerialize(object anything)
      {
         int rawsize = Marshal.SizeOf(anything);
         IntPtr buffer = Marshal.AllocHGlobal(rawsize);
         Marshal.StructureToPtr(anything, buffer, false);
         byte[] rawdatas = new byte[rawsize];
         Marshal.Copy(buffer, rawdatas, 0, rawsize);
         Marshal.FreeHGlobal(buffer);
         return rawdatas;
      }

      //This function is the opposite, it will take an array of bytes, and turn it back into the structure (a typecast is needed although)
      public static object RawDeserialize(byte[] rawdatas, Type anytype)
      {
         int rawsize = Marshal.SizeOf(anytype);
         if (rawsize > rawdatas.Length)
            return null;
         IntPtr buffer = Marshal.AllocHGlobal(rawsize);
         Marshal.Copy(rawdatas, 0, buffer, rawsize);
         object retobj = Marshal.PtrToStructure(buffer, anytype);
         Marshal.FreeHGlobal(buffer);
         return retobj;
      }

      private void CloseEventFile()
      {
         if( m_outFile != null && m_outWriter != null )
         {
            // Seek to beginning and rewrite correct header
            m_outFile.Seek( 0, System.IO.SeekOrigin.Begin );
            byte[] arBytes = RawSerialize(m_FileHeader);
            m_outWriter.Write(arBytes);
            m_outFile.Close();
            m_outFile = null;
            m_outWriter.Close();
            m_outWriter = null;
         }
      }

        #region VSTO generated code

        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InternalStartup()
        {
            this.Startup += new System.EventHandler(ThisAddIn_Startup);
            this.Shutdown += new System.EventHandler(ThisAddIn_Shutdown);
        }
        
        #endregion
    }
}
